/*
 * Copyright (C) Sat Mon May 24 2004 John Watson
 * john@flagrantdisregard.com
 * http://flagrantdisregard.com/headline
 *
 * $Id: BlogFactory.java,v 1.4 2004/06/22 07:31:04 yafd Exp $
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

/*
 * This class is a factory for Blog objects.  Given some blog arguments, it
 * creates the appropriate concrete Blog object.  To add a new blog api, you
 * should just need to add it to BlogType.java, create a checkbox for it in
 * HeadlineBlogOptionPane.java, and of course implement it as a child of the
 * abstract Blog.
 */

// java
import java.lang.reflect.*;
import java.util.Vector;

// swing
import javax.swing.*;

// jedit
import org.gjt.sp.util.Log;

public class BlogFactory {
    public static Blog createInstance(String blogName, String hostName, int port, String endPoint, String username, String password, String api) {
        Blog b = null;
        try {
            Class blogClass = Class.forName(api);
            Constructor blogConstructor = blogClass.getConstructors()[0];
            Vector args = new Vector();
            args.add(blogName);
            args.add(hostName);
            args.add(new Integer(port));
            args.add(endPoint);
            args.add(username);
            args.add(password);
            args.add(api);
            b = (Blog)blogConstructor.newInstance(args.toArray());
            Log.log(Log.MESSAGE, Headline.class, "Creating instance of " + api + " API for " + b.getName());
            return(b);
        } catch (ClassNotFoundException ex) {
            JOptionPane.showMessageDialog(null, "Unknown API: " + api);
            Log.log(Log.MESSAGE, Headline.class, "Unknown API: " + api + "\n" + ex);
            b = null;
        } catch (InstantiationException ex) {
            JOptionPane.showMessageDialog(null, "Unknown API: " + api);
            Log.log(Log.MESSAGE, Headline.class, "Unknown API: " + api + "\n" + ex);
            b = null;
        } catch (IllegalAccessException ex) {
            Log.log(Log.MESSAGE, Headline.class, ex);
            b = null;
        } catch (InvocationTargetException ex) {
            Log.log(Log.MESSAGE, Headline.class, ex);
            b = null;
        }
        return(b);
    }
}

