/*
 * Copyright (C) Sat May 15 2004 John Watson
 * john@flagrantdisregard.com
 * http://flagrantdisregard.com/headline
 *
 * $Id: BloggerBlog.java,v 1.8 2004/06/26 19:48:54 yafd Exp $
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

/*
 * Blog that supports the BloggerAPI
 * http://www.blogger.com/developers/api/1_docs/
 */

// java
import java.awt.Component;
import java.net.*;
import java.io.*;
import javax.swing.*;
import java.lang.StringBuffer;

// xml
import javax.xml.parsers.*;
import javax.xml.transform.*;
import javax.xml.transform.dom.*;
import javax.xml.transform.stream.*;
import org.w3c.dom.*;
import org.xml.sax.*;

// jedit
import org.gjt.sp.jedit.*;
import org.gjt.sp.jedit.gui.*;
import org.gjt.sp.jedit.io.*;
import org.gjt.sp.util.*;
import org.gjt.sp.jedit.msg.*;
import org.gjt.sp.util.Log;

public class BloggerBlog extends Blog {
    private static final int MODE_ADD = 1;
    private static final int MODE_UPDATE = 2;
    private static final int MODE_DELETE = 3;
    
    public BloggerBlog(String blogName, String hostName, int port, String endPoint, String username, String password, String api) {
        super(blogName, hostName, port, endPoint, username, password, api);
    }
    
    public String newPost(String content, boolean publish) {
        String blogId;
        String post_id = null;
        
        // Choose a blog belonging to this user
        blogId = getAvailableBlogs();
        
        // Update blog
        if (blogId != null) {
            post_id = update(MODE_ADD, blogId, content, publish);
        }
        
        return(post_id);
    }
    
    public String editPost(String content, String postId, boolean publish) {
        return(update(MODE_UPDATE, postId, content, publish));
    }
    
    public String deletePost(String postId, boolean publish) {
        return(update(MODE_DELETE, postId, null, publish));
    }
    
    public String getRecentPosts(int number) {
        StringBuffer request = new StringBuffer();
        String response;
        String fault;
        String blogId;
        String postId = null;
        
        promptForPassword();
        
        class Post {
            public String authorName;
            public String userId;
            public String dateCreated;
            public String content;
            public String postId;
            public String category;
            
            public Post() {
                this.authorName = null;
                this.userId = null;
                this.dateCreated = null;
                this.content = null;
                this.postId = null;
                this.category = null;
            }
            
            public String toString() {
                if (content.length() > 49) {
                    return(content.substring(0,49));
                } else {
                    return(content);
                }
            }
        }
        
        // Choose a blog belonging to this user
        blogId = getAvailableBlogs();
        
        // Update blog
        if (blogId != null) {
            request.append("<?xml version=\"1.0\"?>");
            request.append("<methodCall>");
            request.append("<methodName>blogger.getRecentPosts</methodName>");
            request.append("<params>");
            request.append("<param><value><string>").append(APP_KEY).append("</string></value></param>");
            request.append("<param><value><string>").append(blogId).append("</string></value></param>");
            request.append("<param><value><string>").append(username).append("</string></value></param>");
            request.append("<param><value><string>").append(password).append("</string></value></param>");
            request.append("<param><value><int>").append(String.valueOf(number)).append("</int></value></param>");
            request.append("</params>");
            request.append("</methodCall>");
            response = connect(request.toString());
            if (response == null) {
                return(null);
            }
            fault = getFault(response);
            if (fault != null) {
                JOptionPane.showMessageDialog(null, "The server returned an error:\n" + fault + "\nSee activity log for details.");
                Log.log(Log.MESSAGE, Headline.class, "Server fault: " + response);
                return(null);
            }
            
            // Load and parse xml response
            Document doc = null;
            DocumentBuilder db = null;
            DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
            factory.setValidating(false);
            
            try {
                db = factory.newDocumentBuilder();
                doc = db.parse(new ByteArrayInputStream(response.getBytes()));
            } catch (IOException ex) {
                JOptionPane.showMessageDialog(null, "An error occurred trying to\nparse the response from the server");
                Log.log(Log.MESSAGE, Headline.class, "Could not parse response: " + ex);
                return(null);
            } catch (ParserConfigurationException ex) {
                JOptionPane.showMessageDialog(null, "An error occurred trying to\nparse the response from the server");
                Log.log(Log.MESSAGE, Headline.class, "Could not parse response: " + ex);
                return(null);
            } catch (SAXException ex) {
                JOptionPane.showMessageDialog(null, "An error occurred trying to\nparse the response from the server");
                Log.log(Log.MESSAGE, Headline.class, "Could not parse response: " + ex);
                return(null);
            }
            
            Post[] post = null;
            
            try {
                NodeList postStruct = doc.getDocumentElement().getElementsByTagName("struct");
                if (postStruct.getLength() <= 0) {
                    JOptionPane.showMessageDialog(null, "No posts.");
                    return(null);
                }
                
                post = new Post[postStruct.getLength()];
                
                for (int i=0; i<postStruct.getLength(); i++) {
                    NodeList member = postStruct.item(i).getChildNodes();
                    
                    post[i] = new Post();
                    
                    for(int j=0; j<member.getLength(); j++) {
                        NodeList children = member.item(j).getChildNodes();
                        
                        String name = XMLHelper.NodeText(children.item(0));
                        String value = XMLHelper.NodeText(children.item(1));
                        
                        if (name.compareTo("authorName") == 0) {
                            post[i].authorName = value;
                        }
                        
                        if (name.compareTo("userid") == 0) {
                            post[i].userId = value;
                        }
                        
                        if (name.compareTo("dateCreated") == 0) {
                            post[i].dateCreated = value;
                        }
                        
                        if (name.compareTo("content") == 0) {
                            post[i].content = XMLHelper.htmlDecode(value);
                        }
                        
                        if (name.compareTo("postid") == 0) {
                            post[i].postId = value;
                        }
                        
                        if (name.compareTo("category") == 0) {
                            post[i].category = value;
                        }
                    }
                }
            } catch (Exception ex) {
                JOptionPane.showMessageDialog(null, "An error occurred while parsing the post list");
                Log.log(Log.MESSAGE, Headline.class, "An error occurred while parsing the post list: " + ex);
                return(null);
            }
            
            if (post != null) {
                Post p = null;
                
                // If the number of posts is just 1, then just return that;
                // otherwise show a list of posts to choose from.
                if (post.length == 1) {
                    p = post[0];
                } else {
                    p = (Post)JOptionPane.showInputDialog(
                        null,
                        "Post: ",
                        "Choose a post",
                        JOptionPane.PLAIN_MESSAGE,
                        null,
                        post,
                        null);
                }
                
                if (p != null) {
                    postId = p.postId;
                    
                    // Get content into buffer
                    Buffer buf = jEdit.newFile(Headline.view);
                    buf.insert(0, p.content);
                    
                    // Set buffer-local properties
                    Log.log(Log.MESSAGE, Headline.class, "Setting buffer-local " + HeadlinePlugin.PROPERTY_TEMP_BLOGSERVICE + ": " + this.blogName);
                    Log.log(Log.MESSAGE, Headline.class, "Setting buffer-local " + HeadlinePlugin.PROPERTY_TEMP_POSTID + ": " + p.postId);
                    buf.setProperty(HeadlinePlugin.PROPERTY_TEMP_BLOGSERVICE, this.blogName);
                    buf.setProperty(HeadlinePlugin.PROPERTY_TEMP_POSTID, p.postId);
                }
                
            } else {
                JOptionPane.showMessageDialog(null, "No posts found.");
                return(null);
            }
        }
        
        return(postId);
    }
    
    private String update(int mode, String id, String content, boolean publish) {
        StringBuffer request = new StringBuffer();
        String response;
        String fault;
        
        promptForPassword();
        
        request.append("<?xml version=\"1.0\"?>");
        request.append("<methodCall>");
        switch(mode) {
            case MODE_ADD:
                request.append("<methodName>blogger.newPost</methodName>");
                break;
            case MODE_UPDATE:
                request.append("<methodName>blogger.editPost</methodName>");
                break;
            case MODE_DELETE:
                request.append("<methodName>blogger.deletePost</methodName>");
                break;
        }
        request.append("<params>");
        request.append("<param><value><string>").append(APP_KEY).append("</string></value></param>");
        request.append("<param><value><string>").append(id).append("</string></value></param>");
        request.append("<param><value><string>").append(username).append("</string></value></param>");
        request.append("<param><value><string>").append(password).append("</string></value></param>");
        if (mode != MODE_DELETE) {
            request.append("<param><value><string>").append(XMLHelper.htmlEncode(content)).append("</string></value></param>");
        }
        if (publish) {
            request.append("<param><value><boolean>1</boolean></value></param>");
        } else {
            request.append("<param><value><boolean>0</boolean></value></param>");
        }
        request.append("</params>");
        request.append("</methodCall>");
        response = connect(request.toString());
        if (response == null) {
            return(null);
        }
        fault = getFault(response);
        if (fault != null) {
            JOptionPane.showMessageDialog(null, "The server returned an error:\n" + fault + "\nSee activity log for details.");
            Log.log(Log.MESSAGE, Headline.class, "Server fault: " + response);
            return(null);
        }
        
        JOptionPane.showMessageDialog(null, "Blog updated");
        
        return(null);
    };
}

