/*
 * Copyright (C) October 9, 2002 John Watson
 * john@flagrantdisregard.com
 * http://flagrantdisregard.com/headline
 *
 * $Id: Headline.java,v 1.15 2004/06/24 07:32:42 yafd Exp $
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

// java:
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.net.*;
import java.util.Vector;
import java.util.Date;
import java.net.URL;
import java.util.*;

// swing:
import javax.swing.*;
import javax.swing.event.*;

// xml
import javax.xml.parsers.*;
import javax.xml.transform.*;
import javax.xml.transform.dom.*;
import javax.xml.transform.stream.*;
import org.w3c.dom.*;
import org.xml.sax.*;

// jedit:
import org.gjt.sp.jedit.*;
import org.gjt.sp.jedit.gui.*;
import org.gjt.sp.jedit.io.*;
import org.gjt.sp.util.*;
import org.gjt.sp.jedit.msg.*;
import org.gjt.sp.util.Log;

public class Headline extends JPanel implements MouseListener, Runnable, EBComponent {
	private boolean floating;
	public static View view;
	private volatile Thread headlineThread = null;
	private static boolean isCached = false;
	private String textContent = null;
	
	// parseXML modes
	public static final boolean VALIDATE	= true;
	public static final boolean NOVALIDATE	= false;
	public static final boolean CONFIGMODE	= true;
	public static final boolean URLMODE		= false;
	public static final boolean CACHE		= true;
	public static final boolean NOCACHE		= false;
	
	// GUI
	private JPanel controlPanel = null;
	private JPanel buttonPanel = null;
	private JPanel toolBar = null;
	private JPanel comboPanel = null;
	private JComboBox contentList = null;
	private JPanel titlePanel = null;
	private JScrollPane titleScrollPane = null;
	private JPanel contentPanel = null;
	private JScrollPane contentScrollPane = null;
	private JSplitPane splitPane = null;
	private JMenuItem menuItemRefresh = new JMenuItem(jEdit.getProperty("HeadlinePlugin.update-headlines.label"));
	private JMenuItem menuItemTextToBuffer = new JMenuItem(jEdit.getProperty("HeadlinePlugin.text-to-buffer.label"));
	private JMenuItem menuItemXMLToBuffer = new JMenuItem(jEdit.getProperty("HeadlinePlugin.xml-to-buffer.label"));
	//private JMenuItem menuItemClear = new JMenuItem(jEdit.getProperty("HeadlinePlugin.clear-and-refresh.label"));
	private JButton btnRefresh = null;
	private JButton btnTXT2Buf = null;
	private JButton btnXML2Buf = null;
	private JButton btnClear = null;
	private JButton btnNext = null;
	private JButton btnPrev = null;
	
	// actions
	private ActionListener actionUpdate = null;
	private ActionListener actionCopyText = null;
	private ActionListener actionCopyXML = null;
	private ActionListener actionClear = null;
	
	// list of stories
	private ArrayList stories = new ArrayList();
	private int storyIndex = 0;
	
	public void start() {
		if (headlineThread == null) {
			headlineThread = new Thread(this);
			headlineThread.start();
		} else {
			try {
				// .stop() is deprecated
				// http://java.sun.com/j2se/1.4.1/docs/guide/misc/threadPrimitiveDeprecation.html
				//headlineThread.stop();
				headlineThread.interrupt();
				headlineThread = null;
				titlePanel.removeAll();
				titlePanel.repaint();
				contentPanel.removeAll();
				contentPanel.repaint();
				toolbarEnabled(true);
			} catch (Exception ex) {}
		}
	}
	
	public void run() {
		if (headlineThread != null) {
			// Sets the context ClassLoader for this Thread.
			// The context ClassLoader can be set when a thread is created, and allows the creator of
			// the thread to provide the appropriate class loader to code running in the thread when
			// loading classes and resources.  This is to avoid java.lang.ClassCastException errors
			// from being thrown when doing XML transformations.  There is some competition going on
			// between JAXP and xerces (the XML plugin).
			//headlineThread.currentThread().setContextClassLoader(HeadlinePlugin.class.getClassLoader());
			
			// update the headlines
			updateHeadlines();
			
			// stop the thread
			headlineThread = null;
		}
	}
	
	public Headline(View view, String position) {
		super();
		
		// Store view in this object
		this.view = view;
		
		// Set window layout and size
		this.setLayout(new BorderLayout());
		this.floating = position.equals(DockableWindowManager.FLOATING);
		
		// action listeners
		actionUpdate = new ActionListener() {
			public void actionPerformed(ActionEvent event) {
				start();
			}
		};
		actionCopyText = new ActionListener() {
			public void actionPerformed(ActionEvent event) {
				copyTextToNewBuffer();
			}
		};
		actionCopyXML = new ActionListener() {
			public void actionPerformed(ActionEvent event) {
				copyXMLToNewBuffer();
			}
		};
		actionClear =  new ActionListener() {
			public void actionPerformed(ActionEvent event) {
				clearCacheAndRefresh();
			}
		};
		
		EditBus.addToBus(this);
		
		// panel to hold combo box
		comboPanel = new JPanel(new BorderLayout());
		//comboPanel.setLayout(new BoxLayout(comboPanel, BoxLayout.Y_AXIS));
		
		// toolbar buttons
		btnRefresh = new JButton(GUIUtilities.loadIcon("Reload.png"));
		btnRefresh.setBorderPainted(false);
		btnRefresh.setToolTipText(jEdit.getProperty("HeadlinePlugin.update-headlines.label"));
		btnRefresh.setPreferredSize(new Dimension(24,24));
		/*btnRefresh.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent evt) {
				start();
			}
		});*/
		btnRefresh.addActionListener(actionClear);
		
		btnNext = new JButton(GUIUtilities.loadIcon("ArrowR.png"));
		btnNext.setBorderPainted(false);
		btnNext.setToolTipText(jEdit.getProperty("HeadlinePlugin.next.label"));
		btnNext.setPreferredSize(new Dimension(20,16));
		btnNext.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent evt) {
				nextFeed();
			}
		});
		
		btnPrev = new JButton(GUIUtilities.loadIcon("ArrowL.png"));
		btnPrev.setBorderPainted(false);
		btnPrev.setToolTipText(jEdit.getProperty("HeadlinePlugin.previous.label"));
		btnPrev.setPreferredSize(new Dimension(20,16));
		btnPrev.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent evt) {
				previousFeed();
			}
		});
		
		// panel to hold buttons
		buttonPanel = new JPanel();
		buttonPanel.add(btnRefresh);
		buttonPanel.add(btnPrev);
		buttonPanel.add(btnNext);
		buttonPanel.setMinimumSize(new Dimension(64,24));
		
		// panel to hold buttonpanel and combobox
		controlPanel = new JPanel(new BorderLayout());
		controlPanel.add(comboPanel, BorderLayout.CENTER);
		controlPanel.add(buttonPanel, BorderLayout.WEST);

		// load feeds
		refreshContentList();
        
        // load blogs
		HeadlineBlogOptionPane.loadConfig();
        
		// Create content panel and scroll panel for titles
		titlePanel = new JPanel();
		titlePanel.setLayout(new BoxLayout(titlePanel, BoxLayout.Y_AXIS));
		titlePanel.setBackground(Color.WHITE);
		titlePanel.setBorder(BorderFactory.createEmptyBorder(5,5,5,5));
		titleScrollPane = new JScrollPane(titlePanel);
		titleScrollPane.setWheelScrollingEnabled(true);
		titleScrollPane.getVerticalScrollBar().setUnitIncrement(titlePanel.getFontMetrics(titlePanel.getFont()).getHeight());
		titleScrollPane.getHorizontalScrollBar().setUnitIncrement(titlePanel.getFontMetrics(titlePanel.getFont()).stringWidth("W"));
		titleScrollPane.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
		
		// Create content panel and scroll panel for content
		contentPanel = new JPanel();
		contentPanel.setLayout(new BoxLayout(contentPanel, BoxLayout.Y_AXIS));
		contentPanel.setBackground(Color.WHITE);
		contentPanel.setBorder(BorderFactory.createEmptyBorder(5,5,5,5));
		contentScrollPane = new JScrollPane(contentPanel);
		contentScrollPane.setWheelScrollingEnabled(true);
		contentScrollPane.getVerticalScrollBar().setUnitIncrement(contentPanel.getFontMetrics(contentPanel.getFont()).getHeight());
		contentScrollPane.getHorizontalScrollBar().setUnitIncrement(titlePanel.getFontMetrics(contentPanel.getFont()).stringWidth("W"));
		contentScrollPane.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
		
		// Create split pane
		splitPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT, titleScrollPane, contentScrollPane);
		splitPane.setOneTouchExpandable(true);
		splitPane.setContinuousLayout(true);
		
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				int loc = jEdit.getIntegerProperty(HeadlinePlugin.PROPERTY_DIVIDER, -1);
				if (loc == -1) {
					splitPane.setDividerLocation(0.65);
				} else {
					splitPane.setDividerLocation(loc);
				}
			}
		});
		
		// Add the components to the window
		this.add(controlPanel, BorderLayout.NORTH);
		this.add(splitPane, BorderLayout.CENTER);
		
		// popup menu items
		menuItemRefresh.addActionListener(actionClear);
		menuItemTextToBuffer.addActionListener(actionCopyText);
		menuItemXMLToBuffer.addActionListener(actionCopyXML);
		//menuItemClear.addActionListener(actionClear);
	}
	
	private void toolbarEnabled(boolean enabled) {
		contentList.setEnabled(enabled);
		btnNext.setEnabled(enabled);
		btnPrev.setEnabled(enabled);
		if (enabled) {
			btnRefresh.setIcon(GUIUtilities.loadIcon("Reload.png"));
			btnRefresh.setToolTipText(jEdit.getProperty("HeadlinePlugin.update-headlines.label"));
		} else {
			btnRefresh.setIcon(GUIUtilities.loadIcon("Cancel.png"));
			btnRefresh.setToolTipText("Cancel");
		}
	}
	
	public void handleMessage(EBMessage message) {
		if (message instanceof PropertiesChanged) {
            // reload feeds
			refreshContentList();
		}
		
		if (message instanceof EditorExiting) {
			// Save split pane location
			jEdit.setIntegerProperty(HeadlinePlugin.PROPERTY_DIVIDER, (int)splitPane.getDividerLocation());
		}
	}

	public void refreshContentList() {
		// load feed list
		HeadlineOptionPane.loadConfig();
		
		comboPanel.removeAll();
		
		// remove action listeners
		try {
			contentList.removeActionListener(actionUpdate);
		} catch (Exception ex) {}
		
		// Create content combobox
		if (HeadlineOptionPane.feeds != null) {
			contentList = new JComboBox(HeadlineOptionPane.feeds);
		} else {
			contentList = new JComboBox();
		}
		contentList.addActionListener(actionUpdate);
		comboPanel.add(contentList, BorderLayout.CENTER);
		comboPanel.validate();
	}
	
	private void updateHeadlines() {
		if (HeadlineOptionPane.feeds == null) { return; }
		
		String name = "", tmpString = "";
		String content_title = "",
			content_link = "",
			content_description = "",
			content_description_plain = "",
			content_image = "",
			content_date = "";
		String default_link = "";
		int words = 0, words_per_line = 0, lines = 0, avg_word_size = 0;
		NodeList story, storychild;
		int index = contentList.getSelectedIndex();
		String url = HeadlineOptionPane.feeds[index].getUrl();
		Document doc = null;
		LinkLabel storylabel = null;
		String errmsg = null;
		
		if (url == null) { return; }
		
		// disable toolbar
		toolbarEnabled(false);
		
		// clear the windows
		titlePanel.removeAll();
		titlePanel.repaint();
		
		contentPanel.removeAll();
		contentPanel.repaint();
		
		titlePanel.add(new JLabel("Loading..."));
		titlePanel.validate();
		
		// Load the xml file
		try {
			doc = parseXMLFile(url, NOVALIDATE, URLMODE);
		} catch (SAXException ex) {
			errmsg = "The feed does not appear to be valid XML.  The specific error message is: <i>" + ex.getMessage() + "</i>  The problem is probably temporary.  Try again later.";
		} catch (ParserConfigurationException ex) {
			errmsg = ex.toString();
		} catch (IOException ex) {
			if (ex instanceof UnknownHostException) {
				errmsg = "The site could not be reached.  It may be temporarily offline.  Try again later.";
			} else {
				errmsg = ex.toString();
			}
		}
		
		// check for thread stopped
		if (headlineThread == null) { return; }
		
		// clear the window
		titlePanel.removeAll();
		titlePanel.repaint();
		
		if (doc == null) {
			storylabel = new LinkLabel("<html><body><b>Could not open feed.</b><br>" + errmsg + "</body></html>");
			storylabel.setPreferredSize(new Dimension(10, 10));
			storylabel.setVerticalAlignment(JLabel.TOP);
			titlePanel.add(storylabel);
			toolbarEnabled(true);
			titleScrollPane.invalidate();
			this.validate();
			return;
		}
		
		// erase text content
		textContent = "";
		
		// add freshness seal
		int age = (int)cacheAge(url);
		if (age < 1) {
			titlePanel.add(new JLabel("<html><body><i>Content refreshed</i></body></html>"));
		} else {
			titlePanel.add(new JLabel("<html><body><i>Cached: " + new Integer(age).toString() + " seconds ago</i></body></html>"));
		}
		titlePanel.add(Box.createVerticalStrut(10));
		
		// get default link
		try {
			Node tmp = null;
			NodeList tmplist = doc.getDocumentElement().getElementsByTagName("channel").item(0).getChildNodes();
			for (int i=0;i<tmplist.getLength();i++) {
				name = tmplist.item(i).getNodeName();
				if (name.equalsIgnoreCase("link")) {
					default_link = tmplist.item(i).getChildNodes().item(0).getNodeValue();
					break;
				}
			}
		} catch (Exception ex) {
			default_link = "";
		}
		
		// iterate through story elements
		int storycount = 0;
		
		// reset story pointer when the feed changes
		stories.clear();
		storyIndex = 0;
		
        boolean atom = false;
		try {
			story = doc.getDocumentElement().getElementsByTagName("item");
            if (story.getLength() <= 0) {
                // Try Atom (What is Atom? http://help.blogger.com/bin/answer.py?answer=697&topic=36)
                story = doc.getDocumentElement().getElementsByTagName("entry");
                atom = true;
                Log.log(Log.MESSAGE, Headline.class, "This is an atom feed.");
            }
			for (int i=0; i<story.getLength(); i++) {
				storycount++;
				
				// iterate through children of storychild
				storychild = story.item(i).getChildNodes();
				for (int j=0; j<storychild.getLength(); j++) {
                    name = storychild.item(j).getNodeName();
                    
                    if (name.equalsIgnoreCase("title")) {
                        try {
                            content_title = storychild.item(j).getChildNodes().item(0).getNodeValue();
                        } catch (Exception ex) {
                            Log.log(Log.MESSAGE, Headline.class, "Error parsing " + name + ": " + ex);
                        }
                    }
                    
                    if (name.equalsIgnoreCase("description") || name.equalsIgnoreCase("content")) {
                        try {
                            if (atom) {
                                Node mode = storychild.item(j).getAttributes().getNamedItem("mode");
                                NodeList children = storychild.item(j).getChildNodes();
                                if (children.getLength() > 1 && (mode == null || mode != null && mode.getNodeValue().equalsIgnoreCase("xml"))) {
                                    content_description = XMLHelper.NodesToString(storychild.item(j).getChildNodes());
                                }
                            }
                                
                            if (content_description.length() == 0) {
                                content_description = storychild.item(j).getChildNodes().item(0).getNodeValue();
                            }
                            
                            content_description_plain = stripTags(content_description);
                            if (jEdit.getBooleanProperty(HeadlinePlugin.PROPERTY_STRIPTAGS)) {
                                content_description = content_description_plain;
                            }
                        } catch (Exception ex) {
                            Log.log(Log.MESSAGE, Headline.class, "Error parsing " + name + ": " + ex);
                        }
                    }
                    
                    if (name.equalsIgnoreCase("link")) {
                        if (atom) {
                            try {
                                if (storychild.item(j).getAttributes().getNamedItem("rel").getNodeValue().equalsIgnoreCase("alternate")) {
                                    content_link = storychild.item(j).getAttributes().getNamedItem("href").getNodeValue();
                                }
                            } catch (Exception ex) {
                                Log.log(Log.MESSAGE, Headline.class, "Error parsing " + name + ": " + ex);
                            }
                        } else {
                            try {
                                content_link = storychild.item(j).getChildNodes().item(0).getNodeValue();
                            } catch (Exception ex) {
                                Log.log(Log.MESSAGE, Headline.class, "Error parsing " + name + ": " + ex);
                            }
                        }
                    }
                    
                    if (name.equalsIgnoreCase("image")) {
                        try {
                            content_image = storychild.item(j).getChildNodes().item(0).getNodeValue();
                        } catch (Exception ex) {
                            Log.log(Log.MESSAGE, Headline.class, "Error parsing " + name + ": " + ex);
                        }
                    }
                    
                    if (name.indexOf("date") >= 0 || name.indexOf("Date") >= 0 || name.indexOf("created") >= 0 || name.indexOf("issued") >= 0) {
                        try {
                            content_date = storychild.item(j).getChildNodes().item(0).getNodeValue();
                        } catch (Exception ex) {
                            Log.log(Log.MESSAGE, Headline.class, "Error parsing " + name + ": " + ex);
                        }
                    }
				}
				
				// default values
				if (content_title == "") {
					content_title = "No title";
				}
				
				if (content_date == "") {
					content_date = "No date";
				}
				
				if (content_description == "") {
					content_description = "No content";
				}
				
				// build output string
				tmpString = "";
				if (content_title != "") {
					if (tmpString != "") { tmpString += "<br>"; }
					tmpString += "<b>" + content_title + "</b> ";
					textContent += content_title.replaceAll("\n", "").replaceAll("\t", "").trim() + "\n";
				}
				if (content_date != "") {
					if (tmpString != "") { tmpString += "<br>"; }
					tmpString += "<i>" + content_date + "</i> ";
					textContent += content_date.replaceAll("\n", "").replaceAll("\t", "").trim() + "\n";
				}
				if (content_description != "") {
					textContent += content_description.replaceAll("\n", "").replaceAll("\t", "").trim() + "\n";
				}
				tmpString = "<html><body><p>" + tmpString + "</p></body></html>";
				if (textContent != "") { textContent += "\n"; }
				
				// Add content
				storylabel = new LinkLabel(tmpString);
				if (content_link != "") {
					storylabel.url = content_link;
				} else if (default_link != "") {
					storylabel.url = default_link;
				} else {
					storylabel.url = "";
				}
				if (content_description != "") {
					// the replaceAll()s are there because the built-in Java HTML renderer
					// doesn't seem to understand closed tags <... />
					storylabel.content += content_description.replaceAll("/>", ">") + "\n";
				}
				
				// get font sizes
				FontMetrics fm = storylabel.getFontMetrics(storylabel.getFont());
				int fontheight = fm.getHeight();
				int fontwidth = fm.stringWidth("W");
				
				// This is the second best layout so far
				//storylabel.setPreferredSize(new Dimension(10, fontheight*3));
				
				// This is the best layout so far
				Dimension d = splitPane.getSize();
				titlePanel.setMaximumSize(d);
				storylabel.setMinimumSize(new Dimension(10,10));
				storylabel.setMaximumSize(new Dimension((short)d.getWidth()-titleScrollPane.getVerticalScrollBar().getWidth()-15, Short.MAX_VALUE));
				
				storylabel.setVerticalAlignment(JLabel.TOP);
				storylabel.addMouseListener(this);
				storylabel.setBackground(Color.WHITE);
				storylabel.setOpaque(true); // required for highlighting selected entry
				titlePanel.add(storylabel);
				tmpString = "";
				
				// Draw divider
				if (jEdit.getBooleanProperty(HeadlinePlugin.PROPERTY_DRAWBORDER)) {
					// Add a spacer
					titlePanel.add(Box.createVerticalStrut(5));
					
					JLabel tmp = new JLabel("");
					tmp.setMinimumSize(new Dimension(10,10));
					tmp.setMaximumSize(new Dimension((short)d.getWidth()-titleScrollPane.getVerticalScrollBar().getWidth()-15, 2));
					tmp.setPreferredSize(new Dimension(10, 2));
					tmp.setBorder(BorderFactory.createLineBorder(Color.LIGHT_GRAY, 1));
					titlePanel.add(tmp);
				}
				
				// Add a spacer
				titlePanel.add(Box.createVerticalStrut(5));
				
				// auto-load first entry into the content panel
				if (i==0) {
					updateContentPane(storylabel);
				}
				
				// add story to arraylist
				storylabel.index = i;
				stories.add(storylabel);
				
				// reset for next entry
				content_title = "";
				content_link = "";
				content_description = "";
				content_description_plain = "";
				content_image = "";
				content_date = "";
			}
			titlePanel.add(Box.createVerticalGlue()); // to take up any slack space
		} catch (Exception e) {
			JOptionPane.showMessageDialog(null, "An error occurred while parsing the XML:\n" + e);
		} finally {
			// clean up gracefully
			toolbarEnabled(true);
		}
		
		// this forces a redraw of all components so that the components
		// don't just keep piling up on top of each other.
		titleScrollPane.getVerticalScrollBar().setValue(0); // scroll to top
		this.validate();
		
		// set pointer back to normal
		toolbarEnabled(true);
	}
	
	// add the given content to the content panel
	public void updateContentPane(LinkLabel l) {
		LinkLabel tmp = null;
		
		// clear
		contentPanel.removeAll();
		contentPanel.repaint();
		
		// change background color of all LinkLabels to default
		for (int i=0; i<titlePanel.getComponentCount(); i++) {
			if (titlePanel.getComponent(i) instanceof LinkLabel) {
				tmp = (LinkLabel)titlePanel.getComponent(i);
				tmp.setBackground(Color.WHITE);
			}
		}
		
		// change background color of THIS LinkLabel to highlight color
		l.setBackground(Color.LIGHT_GRAY);
		l.invalidate();
		
		// add new content
		tmp = new LinkLabel("<html><body><p>" + l.content + "</p></body></html>");
		tmp.url = l.url;
		tmp.link = true;
		
		// get font sizes
		FontMetrics fm = tmp.getFontMetrics(tmp.getFont());
		double fontheight = fm.getHeight();
		double fontwidth = fm.stringWidth("W");
		
		// This is the second-best layout so far
		//tmp.setPreferredSize(new Dimension(10, (short)(fontheight*1.5*(fontwidth*(double)stripTags(l.content).length()/(double)d.getWidth()))));
		
		// This is the best layout so far
		Dimension d = splitPane.getSize();
		contentPanel.setMaximumSize(d);
		tmp.setMinimumSize(new Dimension(10,10));
		tmp.setMaximumSize(new Dimension((short)d.getWidth()-contentScrollPane.getVerticalScrollBar().getWidth()-15, Short.MAX_VALUE));
		
		tmp.setVerticalAlignment(tmp.TOP);
		tmp.addMouseListener(this);
		contentPanel.add(tmp);
		
		// scroll to top of content pane
		contentScrollPane.getVerticalScrollBar().setValue(0);
		
		// Scroll to keep the currently selected story in view
		// The idea is to keep the selected story in view in the titlescrollpane when
		// using the keyboard controls to select stories.  Broken on purpose so you would
		// remember where you are at.
		titleScrollPane.getVerticalScrollBar().setValue(l.getY()-titleScrollPane.getHeight()/2);
		
		// redraw
		this.validate();
	}
	
	private int countWords(String s) {
		int words = 0;
		String[] s2 = s.split(" ");
		words = s2.length;
		return(words);
	}
	
	private void openURL(String url) {
		infoviewer.InfoViewerPlugin.openURL(view, url);
	}
	
	public void mouseClicked(MouseEvent e) {
		if (headlineThread != null) { return; }
		
		if (!e.isPopupTrigger()) {
			LinkLabel l = (LinkLabel)e.getSource();
			if (l.url != "" && l.link) {
				openURL(l.url);
			} else {
				storyIndex = l.index;
				updateContentPane(l);
			}
		}
	}
	
	public void mousePressed(MouseEvent e) {
		if (headlineThread != null) { return; }
		
		if (e.isPopupTrigger()) {
			buildPopupMenu().show(e.getComponent(), e.getX(), e.getY());
		}
	}
	
	public void mouseReleased(MouseEvent e) {
		if (headlineThread != null) { return; }
		
		if (e.isPopupTrigger()) {
			buildPopupMenu().show(e.getComponent(), e.getX(), e.getY());
		}
	}
	
	public void mouseEntered(MouseEvent e) {
		if (headlineThread != null) { return; }
		
		LinkLabel l = (LinkLabel)e.getSource();
		if (l.url != "") { l.setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR)); }
	}
	
	public void mouseExited(MouseEvent e) {
		if (headlineThread != null) { return; }
		
		LinkLabel l = (LinkLabel)e.getSource();
		if (l.url != "") { l.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR)); }
	}
	
	private static void cacheDoc(Document doc, String url) {
		// static because it is called from parseXMLFile() which is also static.
		try {
			File outputDirectory = new File(HeadlineOptionPane.getCacheStoreDirectory());
			if (!outputDirectory.exists()) {
				outputDirectory.mkdirs();
			}
			saveXMLtoFile(doc, getCacheName(url));
		} catch (Exception e) {
			JOptionPane.showMessageDialog(null, "Error caching data:\n" + e);
		}
	}
	
	// Saves an XML document to a file
	public static void saveXMLtoFile(Document doc, String filename) {
		try {
			// xerces
			/*
			OutputStream out = new FileOutputStream(getCacheName(url));
			XMLSerializer serializer = new XMLSerializer();
			serializer.setOutputByteStream(out);
			serializer.serialize(doc);
			out.close();
			*/
			
			// JAXP
			// Transform using the "null" transformation and store in a stringwriter.
			/*
			StringWriter tmpout = new StringWriter();
			TransformerFactory tFactory = TransformerFactory.newInstance();
			Transformer transformer = tFactory.newTransformer();
			DOMSource source = new DOMSource(doc);
			StreamResult result = new StreamResult(tmpout);
			transformer.transform(source, result);
			tmpout.close();
			*/
			
			// No XSLT
			StringWriter tmpout = new StringWriter();
			PrintWriter pw = new PrintWriter(tmpout);
			XMLHelper xmh = new XMLHelper(pw);
			xmh.printTree(doc.getDocumentElement(), "");
			pw.close();
			tmpout.close();
			
			// If that worked, then write the results to the filesystem.
			// This buffered approach should help prevent corrupting the output
			// file if an error occurs during the transformation steps.
			Log.log(Log.MESSAGE, Headline.class, "Caching feed to " + filename);
			Writer out = new OutputStreamWriter(new FileOutputStream(filename), "UTF-8");
			out.write(tmpout.toString());
			out.close();
		} catch (Exception e) {
			JOptionPane.showMessageDialog(null, "Error saving XML to file " + filename + ":\n" + e);
		}
	}
	
	// return the age of the cached file in seconds
	private long cacheAge(String url) {
		File cache = new File(getCacheName(url));
		Date dt = new Date();
		
		long retval = (dt.getTime() - cache.lastModified())/1000;
		if (retval < 0) { retval = 0; }
		
		return(retval);
	}
	
	// return true of the cached xml data needs to be refreshed
	private static boolean isCacheStale(String url) {
		File cache = new File(getCacheName(url));
		Date dt = new Date();
		
		return((dt.getTime() - cache.lastModified()) > jEdit.getIntegerProperty(HeadlinePlugin.PROPERTY_CACHELIFE, HeadlinePlugin.DEFAULT_CACHELIFE)*1000);
	}
	
	// get the cache name based on the url
	private static String getCacheName(String url) {
		if (url == null || url == "") { return(""); }
		
		char c = ' ';
		StringBuffer buf = new StringBuffer();
		
		buf.append(HeadlineOptionPane.getCacheStoreDirectory());
		buf.append(File.separator);
		
		// strip out illegal characters
		for(int i=0;i<url.length();i++) {
			c = url.charAt(i);
			if (Character.isLetterOrDigit(c)) {
				buf.append(c);
			}
		}
		
		buf.append(".xml");
		
		return buf.toString();
	}
	
	public void copyTextToNewBuffer() {
		if (textContent != null && textContent != "") {
			copyToNewBuffer(textContent);
		}
	}
	
	public void copyXMLToNewBuffer() {
		String url = HeadlineOptionPane.feeds[contentList.getSelectedIndex()].getUrl();
		
		try {
			File cache = new File(getCacheName(url));
			FileReader fr = new FileReader(cache);
			char[] data = new char[(int)cache.length()];
			fr.read(data);
			String xml = new String(data);
			copyToNewBuffer(xml);
		} catch (FileNotFoundException ex) {
			JOptionPane.showMessageDialog(null, "Cache file not found:\n" + ex);
		} catch (IOException ex) {
			JOptionPane.showMessageDialog(null, "An error occurred reading the cache file:\n" + ex);
		}
	}
	
	private void copyToNewBuffer(String content) {
		Buffer buf = jEdit.newFile(view);
		buf.insert(0, content);
	}
	
	public void clearCacheAndRefresh() {
		String url = HeadlineOptionPane.feeds[contentList.getSelectedIndex()].getUrl();
		
		try {
			File cache = new File(getCacheName(url));
			cache.delete();
			start();
		} catch (Exception ex) {}
	}
	
	public void nextFeed() {
		int n = contentList.getSelectedIndex();
		if (n >= contentList.getItemCount()-1) {
			n = 0; // wrap to beginning
		} else {
			n++;
		}
		contentList.setSelectedIndex(n);
	}
	
	public void previousFeed() {
		int n = contentList.getSelectedIndex();
		if (n <= 0) {
			n = contentList.getItemCount()-1; // wrap to end
		} else {
			n--;
		}
		contentList.setSelectedIndex(n);
	}
	
	public void nextStory() {
		storyIndex += 1;
		if (storyIndex > stories.size()-1) { storyIndex = 0; }
		updateContentPane((LinkLabel)stories.get(storyIndex));
	}
	
	public void previousStory() {
		storyIndex -= 1;
		if (storyIndex < 0) { storyIndex = stories.size()-1; }
		updateContentPane((LinkLabel)stories.get(storyIndex));
	}
	
	public void gotoStoryURL() {
		openURL(((LinkLabel)stories.get(storyIndex)).url);
	}
	
    public void blogThis(boolean publish) {
        Buffer buf = jEdit.getActiveView().getBuffer();
        String postId = buf.getStringProperty(HeadlinePlugin.PROPERTY_TEMP_POSTID);
        String blogService = buf.getStringProperty(HeadlinePlugin.PROPERTY_TEMP_BLOGSERVICE);
        
        Blog b = null;
        if (postId == null || postId.equalsIgnoreCase("")) {
            // Choose a blogging service
            if (HeadlineBlogOptionPane.blogs.length == 1) {
                b = HeadlineBlogOptionPane.blogs[0];
            } else {
                b = (Blog)JOptionPane.showInputDialog(
                    null,
                    "Service: ",
                    "Choose a service",
                    JOptionPane.PLAIN_MESSAGE,
                    null,
                    HeadlineBlogOptionPane.blogs,
                    null);
            }
        } else {
            // For an existing post, find the correct blogging service
            for(int i=0; i<HeadlineBlogOptionPane.blogs.length; i++) {
                if (HeadlineBlogOptionPane.blogs[i].getName().compareTo(blogService) == 0) {
                    b = HeadlineBlogOptionPane.blogs[i];
                    break;
                }
            }
        }
        
        if (b != null) {
            String content = buf.getText(0, buf.getLength());
            if (postId == null || postId.equalsIgnoreCase("")) {
                b.newPost(content, publish);
            } else {
                if (JOptionPane.OK_OPTION == JOptionPane.showConfirmDialog(null, "Editing post " + postId, "Edit", JOptionPane.OK_CANCEL_OPTION)) {
                    b.editPost(content, postId, publish);
                }
            }
        }
    }
    
    public void deletePost(boolean publish) {
        Buffer buf = jEdit.getActiveView().getBuffer();
        String postId = buf.getStringProperty(HeadlinePlugin.PROPERTY_TEMP_POSTID);
        String blogService = buf.getStringProperty(HeadlinePlugin.PROPERTY_TEMP_BLOGSERVICE);
        
        if (postId == null || postId.equalsIgnoreCase("")) {
            JOptionPane.showMessageDialog(null, "You need to select a post first using Get Recent Posts.");
            return;
        }
        
        // Find the correct blogging service
        Blog b = null;
        for(int i=0; i<HeadlineBlogOptionPane.blogs.length; i++) {
            if (HeadlineBlogOptionPane.blogs[i].getName().compareTo(blogService) == 0) {
                b = HeadlineBlogOptionPane.blogs[i];
                break;
            }
        }
        
        if (b != null) {
            String content = buf.getText(0, buf.getLength());
            if (JOptionPane.OK_OPTION == JOptionPane.showConfirmDialog(null, "Delete post " + postId, "Delete", JOptionPane.OK_CANCEL_OPTION)) {
                b.deletePost(postId, publish);
            }
        }
    }
    
    public void getRecentPosts(int number) {
        Blog b = null;
        
        // Choose a blogging service
        if (HeadlineBlogOptionPane.blogs.length == 1) {
            b = HeadlineBlogOptionPane.blogs[0];
        } else {
            b = (Blog)JOptionPane.showInputDialog(
                null,
                "Service: ",
                "Choose a service",
                JOptionPane.PLAIN_MESSAGE,
                null,
                HeadlineBlogOptionPane.blogs,
                null);
        }
        
        if (b != null) {
            Buffer buf = jEdit.getActiveView().getBuffer();
            String content = buf.getText(0, buf.getLength());
            b.getRecentPosts(number);
        }
    }
    
	private JPopupMenu buildPopupMenu() {
		JPopupMenu menu = new JPopupMenu();
		menu.add(menuItemRefresh);
		menu.add(menuItemTextToBuffer);
		menu.add(menuItemXMLToBuffer);
		//menu.add(menuItemClear);
		return menu;
	}
	
	// given a string, strip out all tags and return text only
	private String stripTags(String s) {
		StringBuffer sb = new StringBuffer();
		char c = '\0';
		boolean inside = false;
		
		inside = false;
		for(int i=0;i<s.length();i++) {
			c = s.charAt(i);
			if (c == '<') inside = true;
			if (!inside) sb.append(c);
			if (c == '>') inside = false;
		}
		
		return(sb.toString());
	}
	
	// Parses an XML file and returns a DOM document.
	// If validating is true, the content is validated against the DTD
	// specified in the file.
	public static Document parseXMLFile(String filename, boolean validating, boolean isconfig, boolean allowcache)
	throws SAXException, ParserConfigurationException, IllegalArgumentException, IOException {
		Document doc = null;
		DocumentBuilder db = null;
		
		// Create a builder factory
		DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
		factory.setValidating(validating);
		
		// Create the builder and parse the file
		db = factory.newDocumentBuilder();
		
		if (isconfig) {
			doc = db.parse(new File(filename));
		} else {
			if (isCacheStale(filename) || !allowcache) {
				// the cache is stale; load from source and save
				isCached = false;
				Log.log(Log.MESSAGE, Headline.class, "Fetching feed " + filename);
				doc = db.parse(filename);
				if (allowcache) { cacheDoc(doc, filename); }
			} else {
				// the cache is fresh; load from cache.
				isCached = true;
				Log.log(Log.MESSAGE, Headline.class, "Loading cached feed " + getCacheName(filename));
				doc = db.parse(new File(getCacheName(filename)));
			}
		}
		
		return(doc);
	}
	
	// overloaded allows optional parameters
	public static Document parseXMLFile(String filename, boolean validating, boolean isconfig)
	throws SAXException, ParserConfigurationException, IllegalArgumentException, IOException {
		return(parseXMLFile(filename, validating, isconfig, CACHE));
	}
}

