/*
 * Copyright (C) Tue May 25 2004 John Watson
 * john@flagrantdisregard.com
 * http://flagrantdisregard.com/headline
 *
 * $Id: HeadlineBlogOptionPane.java,v 1.15 2004/06/26 19:48:54 yafd Exp $
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

// java
import java.io.*;
import java.net.*;
import java.awt.*;
import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Vector;
import java.lang.reflect.Array;
import java.util.Arrays;

// swing
import javax.swing.*;

// xml
import javax.xml.parsers.*;
import javax.xml.transform.*;
import javax.xml.transform.dom.*;
import javax.xml.transform.stream.*;
import org.w3c.dom.*;
import org.xml.sax.*;

// jedit
import org.gjt.sp.jedit.jEdit;
import org.gjt.sp.jedit.GUIUtilities;
import org.gjt.sp.jedit.AbstractOptionPane;
import org.gjt.sp.jedit.gui.FontSelector;

public class HeadlineBlogOptionPane extends AbstractOptionPane implements ActionListener {
	public static Document doc = null;
	public static Blog[] blogs = null;
    
	private JComboBox blogList = null;
	private JButton deleteBlog = null;
	private JButton saveBlog = null;
    private JButton newBlog = null;
    
    private JTextField blogName = new JTextField();
    private JTextField hostName = new JTextField();
    private JTextField port = new JTextField();
    private JTextField endPoint = new JTextField();
    private JTextField username = new JTextField();
    private JPasswordField password = new JPasswordField();
    private JRadioButton atomAPI = new JRadioButton("Atom");
    private JRadioButton bloggerAPI = new JRadioButton("Blogger");
    private JRadioButton metaWeblogAPI = new JRadioButton("MetaWeblog");
    
	public HeadlineBlogOptionPane() {
        super("HeadlinePlugin.blogs");
	}
	
	public void _init() {
		//this.setLayout(new BorderLayout());
        
        loadConfig();
        
        // Main panel
		JPanel blogPanel = new JPanel();
        blogPanel.setLayout(new BoxLayout(blogPanel, BoxLayout.Y_AXIS));
		//blogPanel.setBorder(BorderFactory.createTitledBorder("Blogs"));
        
        // Blog combo box
		if (blogs != null) {
			blogList = new JComboBox(blogs);
		} else {
			blogList = new JComboBox();
		}
		blogList.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
                blogSelected();
			}
		});
        
        // Blog input fields
        JPanel inputPanel = new JPanel();
        inputPanel.setLayout(new GridLayout(9,2));
        inputPanel.add(new JLabel("Service: "));
        inputPanel.add(blogList);
        
        inputPanel.add(Box.createVerticalStrut(10)); // vertical spacer
        inputPanel.add(Box.createVerticalStrut(10)); // vertical spacer
        
        inputPanel.add(new JLabel("Name:"));
        inputPanel.add(blogName);
        inputPanel.add(new JLabel("Host:"));
        inputPanel.add(hostName);
        inputPanel.add(new JLabel("Port:"));
        inputPanel.add(port);
        inputPanel.add(new JLabel("End point:"));
        inputPanel.add(endPoint);
        inputPanel.add(new JLabel("Username:"));
        inputPanel.add(username);
        inputPanel.add(new JLabel("Password:"));
        inputPanel.add(password);
        
        ButtonGroup apiGroup = new ButtonGroup();
        apiGroup.add(bloggerAPI);
        apiGroup.add(metaWeblogAPI);
        apiGroup.add(atomAPI);
        JPanel apiPanel = new JPanel();
        apiPanel.add(bloggerAPI);
        apiPanel.add(metaWeblogAPI);
        apiPanel.add(atomAPI);
        inputPanel.add(new JLabel("API:"));
        inputPanel.add(apiPanel);
        
        // Enabled APIs
        bloggerAPI.setEnabled(true);
        metaWeblogAPI.setEnabled(true);
        atomAPI.setEnabled(false);
        blogSelected();
        
        blogPanel.add(inputPanel);
        
        // Blog buttons
        JPanel buttonPanel = new JPanel();
        
        saveBlog = new JButton("Save");
        saveBlog.addActionListener(this);
        buttonPanel.add(saveBlog);
        
        newBlog = new JButton("New");
        newBlog.addActionListener(this);
        buttonPanel.add(newBlog);
        
        deleteBlog = new JButton("Delete");
        deleteBlog.addActionListener(this);
        buttonPanel.add(deleteBlog);
        
        blogPanel.add(buttonPanel);
        
        // Add components
        add(blogPanel);
	}
	
    public void blogSelected() {
        if (blogs != null && blogList.getSelectedIndex() >= 0) {
            blogName.setText(blogs[blogList.getSelectedIndex()].getName());
            hostName.setText(blogs[blogList.getSelectedIndex()].getHostName());
            port.setText(String.valueOf(blogs[blogList.getSelectedIndex()].getPort()));
            endPoint.setText(blogs[blogList.getSelectedIndex()].getEndPoint());
            username.setText(blogs[blogList.getSelectedIndex()].getUsername());
            password.setText(blogs[blogList.getSelectedIndex()].getPassword());
            if (blogs[blogList.getSelectedIndex()].getAPI().compareTo(BlogType.API_BLOGGER) == 0) {
                bloggerAPI.setSelected(true);
            } else {
                bloggerAPI.setSelected(false);
            }
            if (blogs[blogList.getSelectedIndex()].getAPI().compareTo(BlogType.API_ATOM) == 0) {
                bloggerAPI.setSelected(true);
            } else {
                bloggerAPI.setSelected(false);
            }
            if (blogs[blogList.getSelectedIndex()].getAPI().compareTo(BlogType.API_METAWEBLOG) == 0) {
                metaWeblogAPI.setSelected(true);
            } else {
                metaWeblogAPI.setSelected(false);
            }
        }
    }
    
	public void _save() {
		// save blogs
		try {
			File outputDirectory = new File(HeadlineOptionPane.getFileStoreDirectory());
			if (!outputDirectory.exists()) {
				outputDirectory.mkdirs();
			}
			Headline.saveXMLtoFile(doc, getBlogFileStorePath());
		} catch (Exception e) {
			JOptionPane.showMessageDialog(null, "Error saving blogging options:\n" + e);
		}
	}
    
	public static void loadConfig() {
		doc = null;
		
		// file not found (first run or deleted)
		try {
			File outputDirectory = new File(HeadlineOptionPane.getFileStoreDirectory());
			if (!outputDirectory.exists()) {
				outputDirectory.mkdirs();
			}
		} catch (Exception e) {
			JOptionPane.showMessageDialog(null, "Could not create settings folders:\n" + e);
		}
		
        // No blogs.xml file so create one
        if (!(new File(getBlogFileStorePath())).exists()) {
            try {
                Writer out = new OutputStreamWriter(new FileOutputStream(getBlogFileStorePath()));
                out.write("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n<blogs />\n");
                out.close();
            } catch (Exception e) {
                JOptionPane.showMessageDialog(null, "Error creating blog settings file.");
            }
        }
        
		// Load the xml file
		try {
			doc = Headline.parseXMLFile(getBlogFileStorePath(), Headline.NOVALIDATE, Headline.CONFIGMODE);
		} catch (SAXException ex) {
			JOptionPane.showMessageDialog(null, ex.getMessage());
		} catch (ParserConfigurationException ex) {
			JOptionPane.showMessageDialog(null, ex.toString());
		} catch (IOException ex) {
			JOptionPane.showMessageDialog(null, ex.toString());
		}
		
		// read blog list
		if (doc != null) {
			NodeList blog = doc.getDocumentElement().getElementsByTagName("blog");
			if (blog != null) {
				int n = blog.getLength();
				blogs = new Blog[n];
				for(int i=0;i<blog.getLength();i++) {
					NodeList blogschild = blog.item(i).getChildNodes();
					if (blogschild != null) {
                        String host = null;
                        int port = 0;
                        String endPoint = null;
                        String blogname = null;
                        String username = null;
                        String password = null;
                        String api = BlogType.API_BLOGGER;
						for(int j=0;j<blogschild.getLength();j++) {
							if (blogschild.item(j).getNodeName().equalsIgnoreCase("host")) {
								try {
									host = blogschild.item(j).getChildNodes().item(0).getNodeValue();
								} catch (Exception ex) {}
							}
                            
							if (blogschild.item(j).getNodeName().equalsIgnoreCase("port")) {
								try {
									port = Integer.parseInt(blogschild.item(j).getChildNodes().item(0).getNodeValue().replaceAll("\n", "").replaceAll("\t", "").trim());
								} catch (Exception ex) {}
							}
                            
							if (blogschild.item(j).getNodeName().equalsIgnoreCase("endpoint")) {
								try {
									endPoint = blogschild.item(j).getChildNodes().item(0).getNodeValue();
								} catch (Exception ex) {}
							}
                            
							if (blogschild.item(j).getNodeName().equalsIgnoreCase("blogname")) {
								try {
									blogname = blogschild.item(j).getChildNodes().item(0).getNodeValue();
								} catch (Exception ex) {}
							}
                            
							if (blogschild.item(j).getNodeName().equalsIgnoreCase("username")) {
								try {
									username = blogschild.item(j).getChildNodes().item(0).getNodeValue();
								} catch (Exception ex) {}
							}
                            
							if (blogschild.item(j).getNodeName().equalsIgnoreCase("password")) {
								try {
									password = blogschild.item(j).getChildNodes().item(0).getNodeValue();
								} catch (Exception ex) {}
							}
                            
							if (blogschild.item(j).getNodeName().equalsIgnoreCase("api")) {
								try {
									api = blogschild.item(j).getChildNodes().item(0).getNodeValue().replaceAll("\n", "").replaceAll("\t", "").trim();
								} catch (Exception ex) {}
							}
						}
						blogs[i] = BlogFactory.createInstance(blogname, host, port, endPoint, username, password, api);
                        if (blogs[i] == null) {
                            JOptionPane.showMessageDialog(null, "Couldn't create blog instance.");
                        }
					}
				}
                
				if (blogs != null && jEdit.getBooleanProperty(HeadlinePlugin.PROPERTY_SORT)) {
					Arrays.sort(blogs);
				}
			}
		} else {
			// create a new xml document to store the blogs
			try {
				DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
				factory.setValidating(false);
				DocumentBuilder db = factory.newDocumentBuilder();
				doc = db.newDocument();
				Node blogs = doc.createElement("blogs");
				doc.appendChild(blogs);
			} catch (ParserConfigurationException ex) {
				JOptionPane.showMessageDialog(null, "Could not initialize a new blog XML store:\n" + ex);
			}
		}
	}
	
	public void actionPerformed(java.awt.event.ActionEvent e) {
		// delete button
		if (e.getSource().equals(deleteBlog)) {
            // scan through all existing blogs.  if one found with this name,
            // delete it.
            String name = blogs[blogList.getSelectedIndex()].getName();
            NodeList n = doc.getElementsByTagName("blogname");
            if (name != null) {
                for(int i=0; i<n.getLength(); i++) {
                    if (n.item(i).getFirstChild().getNodeValue().replaceAll("\n", "").replaceAll("\t", "").trim().equals(name)) {
                        doc.getDocumentElement().removeChild(n.item(i).getParentNode());
                        
                        break;
                    }
                }
                
                JOptionPane.showMessageDialog(null, "Deleted.");
                
                saveAndReload();
            }
		}
		
		// new button
		if (e.getSource().equals(newBlog)) {
            //blogList.setSelectedIndex(0);
            blogName.setText("");
            hostName.setText("");
            port.setText("");
            endPoint.setText("");
            username.setText("");
            password.setText("");
            bloggerAPI.setSelected(true);
            atomAPI.setSelected(false);
            metaWeblogAPI.setSelected(false);
		}
		
		// save button
		if (e.getSource().equals(saveBlog)) {
            // scan through all existing blogs.  if one found with same name,
            // update that one.  otherwise, add a new one at the end.
            String name = blogName.getText();
            
            if (name == null || name.length() == 0) {
                JOptionPane.showMessageDialog(null, "Name is required");
                return;
            }
            
            if (hostName.getText() == null || hostName.getText().length() == 0) {
                JOptionPane.showMessageDialog(null, "Host is required");
                return;
            }
            
            if (port.getText() == null || port.getText().length() == 0) {
                JOptionPane.showMessageDialog(null, "Port is required");
                return;
            }
            
            if (endPoint.getText() == null || endPoint.getText().length() == 0) {
                JOptionPane.showMessageDialog(null, "Endpoint is required");
                return;
            }
            
            if (username.getText() == null || username.getText().length() == 0) {
                JOptionPane.showMessageDialog(null, "Username is required");
                return;
            }
            
            if (!atomAPI.isSelected() && !metaWeblogAPI.isSelected() && !bloggerAPI.isSelected()) {
                JOptionPane.showMessageDialog(null, "Choose an API");
                return;
            }
            
            if (name != null) {
                Node newnode_blog = doc.createElement("blog");
                Node newnode_name = doc.createElement("blogname");
                Node newnode_host = doc.createElement("host");
                Node newnode_port = doc.createElement("port");
                Node newnode_endpoint = doc.createElement("endpoint");
                Node newnode_username = doc.createElement("username");
                Node newnode_password = doc.createElement("password");
                Node newnode_api = doc.createElement("api");
                
                newnode_name.appendChild(doc.createTextNode(blogName.getText()));
                newnode_host.appendChild(doc.createTextNode(hostName.getText()));
                newnode_port.appendChild(doc.createTextNode(port.getText()));
                newnode_endpoint.appendChild(doc.createTextNode(endPoint.getText()));
                newnode_username.appendChild(doc.createTextNode(username.getText()));
                newnode_password.appendChild(doc.createTextNode(String.valueOf(password.getPassword())));
                if (atomAPI.isSelected()) {
                    newnode_api.appendChild(doc.createTextNode(BlogType.API_ATOM));
                }
                if (bloggerAPI.isSelected()) {
                    newnode_api.appendChild(doc.createTextNode(BlogType.API_BLOGGER));
                }
                if (metaWeblogAPI.isSelected()) {
                    newnode_api.appendChild(doc.createTextNode(BlogType.API_METAWEBLOG));
                }
                
                newnode_blog.appendChild(newnode_name);
                newnode_blog.appendChild(newnode_host);
                newnode_blog.appendChild(newnode_port);
                newnode_blog.appendChild(newnode_endpoint);
                newnode_blog.appendChild(newnode_username);
                newnode_blog.appendChild(newnode_password);
                newnode_blog.appendChild(newnode_api);
                
                boolean found = false;
                NodeList n = doc.getElementsByTagName("blogname");
                for(int i=0; i<n.getLength(); i++) {
                    if (n.item(i).getFirstChild().getNodeValue().replaceAll("\n", "").replaceAll("\t", "").trim().equals(name)) {
                        found = true;
                        doc.getDocumentElement().replaceChild(newnode_blog, n.item(i).getParentNode());
                    }
                }
                
                if (!found) {
                    doc.getDocumentElement().appendChild(newnode_blog);
                }
                
                JOptionPane.showMessageDialog(null, "Saved.");
                
                saveAndReload();
            }
		}
	}
	
	private void saveAndReload() {
		// save and rebuild the dialog based on the new feeds.  this is
		// necessary because JComboBoxes cannot be edited.
		_save();
		this.removeAll();
		_init();
		this.validate();
	}
	
	public static String getBlogFileStorePath() {
		return HeadlineOptionPane.getFileStoreDirectory() + File.separator + getBlogFileStoreName();
	}
	
	public static String getBlogFileStoreName() {
		return HeadlinePlugin.STORAGE_BLOG_FILENAME;
	}
}

