/*
 * Copyright (C) October 9, 2002 John Watson
 * john@flagrantdisregard.com
 * http://flagrantdisregard.com/headline
 *
 * $Id: HeadlineOptionPane.java,v 1.9 2004/06/24 08:54:01 yafd Exp $
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

// java
import java.io.*;
import java.net.*;
import java.awt.*;
import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Vector;
import java.lang.reflect.Array;
import java.util.Arrays;

// swing
import javax.swing.*;

// xml
import javax.xml.parsers.*;
import javax.xml.transform.*;
import javax.xml.transform.dom.*;
import javax.xml.transform.stream.*;
import org.w3c.dom.*;
import org.xml.sax.*;

// jedit
import org.gjt.sp.jedit.jEdit;
import org.gjt.sp.jedit.GUIUtilities;
import org.gjt.sp.jedit.AbstractOptionPane;
import org.gjt.sp.jedit.gui.FontSelector;

public class HeadlineOptionPane extends AbstractOptionPane implements ActionListener {
	public static Document doc = null;
	public static Feed[] feeds = null;
	
	// gui
	private JComboBox feedList = null;
	private JButton deleteFeed = null;
	private JTextField newFeed = null;
	private JButton addFeed = null;
	private JButton updateFeed = null;
	private JButton updateAllFeeds = null;
	private JTextField feedURL = null;
	private JTextArea feedDesc = null;
	private JTextField cacheLife = null;
	//private JTextField textLimit = null;
	private JCheckBox chkStrip = null;
	private JCheckBox chkSort = null;
	private JCheckBox chkHideDesc = null;
	private JCheckBox chkDrawBorder = null;
	
	public HeadlineOptionPane() {
        super("HeadlinePlugin.feeds");
	}
	
	public void _init() {
		this.setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
		this.setAlignmentY(this.TOP_ALIGNMENT);
		
		JPanel existingPanel = new JPanel();
		existingPanel.setLayout(new BorderLayout());
		existingPanel.setBorder(BorderFactory.createTitledBorder("Existing feeds"));
		
		JPanel feedListPanel = new JPanel();
		feedListPanel.setLayout(new BoxLayout(feedListPanel, BoxLayout.Y_AXIS));
		existingPanel.add(feedListPanel, BorderLayout.NORTH);
		
		JPanel existingControlButtonsPanel = new JPanel(new FlowLayout());
		
		JPanel newPanel = new JPanel(new FlowLayout());
		newPanel.setBorder(BorderFactory.createTitledBorder("Add new feed"));
		
		JPanel propPanel = new JPanel();
		propPanel.setLayout(new BoxLayout(propPanel, BoxLayout.Y_AXIS));
		propPanel.setBorder(BorderFactory.createTitledBorder("Options"));
		
		JPanel propPanel2 = new JPanel(new FlowLayout());
		cacheLife = new JTextField(5);
		cacheLife.setText(new Integer(jEdit.getIntegerProperty(HeadlinePlugin.PROPERTY_CACHELIFE, HeadlinePlugin.DEFAULT_CACHELIFE)).toString());
		propPanel2.add(new JLabel("Cache lifetime: "));
		propPanel2.add(cacheLife);
		propPanel2.add(new JLabel(" seconds"));
		propPanel.add(propPanel2);
		
		/*
		JPanel propPanel3 = new JPanel(new FlowLayout());
		textLimit = new JTextField(5);
		textLimit.setText(new Integer(jEdit.getIntegerProperty(HeadlinePlugin.PROPERTY_TEXTLIMIT, HeadlinePlugin.DEFAULT_TEXTLIMIT)).toString());
		propPanel3.add(new JLabel("Description limit: "));
		propPanel3.add(textLimit);
		propPanel3.add(new JLabel(" characters"));
		propPanel.add(propPanel3);
		*/
		
		JPanel propPanel4 = new JPanel(new FlowLayout());
		chkStrip = new JCheckBox("Remove HTML tags");
		chkStrip.setSelected(jEdit.getBooleanProperty(HeadlinePlugin.PROPERTY_STRIPTAGS));
		propPanel4.add(chkStrip);
		chkSort = new JCheckBox("Sort feed list");
		chkSort.setSelected(jEdit.getBooleanProperty(HeadlinePlugin.PROPERTY_SORT));
		propPanel4.add(chkSort);
		chkDrawBorder = new JCheckBox("Divider between items");
		chkDrawBorder.setSelected(jEdit.getBooleanProperty(HeadlinePlugin.PROPERTY_DRAWBORDER));
		propPanel4.add(chkDrawBorder);
		propPanel.add(propPanel4);
		
		loadConfig();
		
		// add gui components
		if (feeds != null) {
			feedList = new JComboBox(feeds);
		} else {
			feedList = new JComboBox();
		}
		feedList.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if (feeds != null) {
					feedURL.setText(feeds[feedList.getSelectedIndex()].getUrl());
					feedDesc.setText(feeds[feedList.getSelectedIndex()].getDescription());
					feedDesc.setCaretPosition(0); // scroll to top
				}
			}
		});
		feedListPanel.add(feedList);
		
		deleteFeed = new JButton("Delete");
		deleteFeed.addActionListener(this);
		existingControlButtonsPanel.add(deleteFeed);
		
		updateFeed = new JButton("Update");
		updateFeed.addActionListener(this);
		existingControlButtonsPanel.add(updateFeed);
		
		updateAllFeeds = new JButton("Update all");
		updateAllFeeds.addActionListener(this);
		existingControlButtonsPanel.add(updateAllFeeds);
		
		if (feeds != null && feedList.getSelectedIndex() >= 0) {
			feedURL = new JTextField(feeds[0].getUrl());
			feedDesc = new JTextArea(feeds[0].getDescription());
		} else {
			feedURL = new JTextField("");
			feedDesc = new JTextArea();
		}
		feedDesc.setCaretPosition(0); // scroll to top
		feedURL.setEditable(false);
		//feedURL.setPreferredSize(new Dimension(100, 1*feedURL.getFontMetrics(feedURL.getFont()).getHeight()));
		//feedURL.setMaximumSize(new Dimension(Short.MAX_VALUE, 1*feedURL.getFontMetrics(feedURL.getFont()).getHeight()));
		feedDesc.setEditable(false);
		feedDesc.setLineWrap(true);
		feedDesc.setWrapStyleWord(true);
		JScrollPane scrollPane = new JScrollPane(feedDesc);
		scrollPane.setMinimumSize(new Dimension(100, 4*feedDesc.getFontMetrics(feedDesc.getFont()).getHeight()));
		scrollPane.setPreferredSize(new Dimension(100, 4*feedDesc.getFontMetrics(feedDesc.getFont()).getHeight()));
		scrollPane.setMaximumSize(new Dimension(Short.MAX_VALUE, Short.MAX_VALUE));
		scrollPane.setWheelScrollingEnabled(true);
		scrollPane.getVerticalScrollBar().setUnitIncrement(feedDesc.getFontMetrics(feedDesc.getFont()).getHeight());
		
		feedListPanel.add(feedURL);
		existingPanel.add(scrollPane, BorderLayout.CENTER);
		existingPanel.add(existingControlButtonsPanel, BorderLayout.SOUTH);
		
		newPanel.add(new JLabel("URL:"));
		
		newFeed = new JTextField(25);
		newPanel.add(newFeed);
		
		addFeed = new JButton("Add");
		addFeed.addActionListener(this);
		newPanel.add(addFeed);
		
		// add panels
		addComponent(existingPanel);
		addComponent(newPanel);
		addComponent(propPanel);
	}
	
	public void _save() {
		// save properties
		jEdit.setProperty(HeadlinePlugin.PROPERTY_CACHELIFE, cacheLife.getText());
		//jEdit.setProperty(HeadlinePlugin.PROPERTY_TEXTLIMIT, textLimit.getText());
		jEdit.setBooleanProperty(HeadlinePlugin.PROPERTY_STRIPTAGS, chkStrip.isSelected());
		jEdit.setBooleanProperty(HeadlinePlugin.PROPERTY_SORT, chkSort.isSelected());
		jEdit.setBooleanProperty(HeadlinePlugin.PROPERTY_DRAWBORDER, chkDrawBorder.isSelected());
		
		// save feeds
		try {
			File outputDirectory = new File(getFileStoreDirectory());
			if (!outputDirectory.exists()) {
				outputDirectory.mkdirs();
			}
			Headline.saveXMLtoFile(doc, getFileStorePath());
		} catch (Exception e) {
			JOptionPane.showMessageDialog(null, "Error saving options:\n" + e);
		}
	}
	
	public static void loadConfig() {
		doc = null;
		
		// file not found (first run or deleted)
		try {
			File outputDirectory = new File(getFileStoreDirectory());
			if (!outputDirectory.exists()) {
				outputDirectory.mkdirs();
			}
		} catch (Exception e) {
			JOptionPane.showMessageDialog(null, "Could not create settings folders:\n" + e);
		}
		
        // No feeds.xml file so create one
        if (!(new File(getFileStorePath())).exists()) {
            try {
                Writer out = new OutputStreamWriter(new FileOutputStream(getFileStorePath()));
                out.write("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n<feeds>\n<feed>\n<name>jEdit Community - Resources for users of the jEdit Text Editor</name>\n<url>http://community.jedit.org/?q=node/feed</url>\n<description>A place for jEdit users to get help with problems and share plugins, macros, and other tools for programming with jEdit.</description>\n</feed>\n<feed>\n<name>freshmeat.net</name>\n<url>http://freshmeat.net/backend/fm-releases.rdf</url>\n<description>freshmeat.net maintains the Web's largest index of Unix and cross-platform open source software. Thousands of applications are meticulously cataloged in the freshmeat.net database, and links to new code are added daily.</description>\n</feed>\n<feed>\n<name>Slashdot</name>\n<url>http://slashdot.org/slashdot.rss</url>\n<description>News for nerds, stuff that matters</description>\n</feed>\n</feeds>\n");
                out.close();
            } catch (Exception e) {
                JOptionPane.showMessageDialog(null, "Error creating feed settings file.");
            }
        }
        
		// Load the xml file
		try {
			doc = Headline.parseXMLFile(getFileStorePath(), Headline.NOVALIDATE, Headline.CONFIGMODE);
		} catch (SAXException ex) {
			JOptionPane.showMessageDialog(null, ex.getMessage());
		} catch (ParserConfigurationException ex) {
			JOptionPane.showMessageDialog(null, ex.toString());
		} catch (IOException ex) {
			JOptionPane.showMessageDialog(null, ex.toString());
		}
		
		// read feed list
		if (doc != null) {
			NodeList feed = doc.getDocumentElement().getElementsByTagName("feed");
			if (feed != null) {
				int n = feed.getLength();
				feeds = new Feed[n];
				for(int i=0;i<feed.getLength();i++) {
					NodeList feedschild = feed.item(i).getChildNodes();
					if (feedschild != null) {
						String name = null;
						String url = null;
						String desc = null;
						for(int j=0;j<feedschild.getLength();j++) {
							if (feedschild.item(j).getNodeName().equalsIgnoreCase("name")) {
								try {
									name = feedschild.item(j).getChildNodes().item(0).getNodeValue();
								} catch (Exception ex) {}
							}
							
							if (feedschild.item(j).getNodeName().equalsIgnoreCase("url")) {
								try {
									url = feedschild.item(j).getChildNodes().item(0).getNodeValue();
								} catch (Exception ex) {}
							}
							
							if (feedschild.item(j).getNodeName().equalsIgnoreCase("description")) {
								try {
									desc = feedschild.item(j).getChildNodes().item(0).getNodeValue();
								} catch (Exception ex) {}
							}
						}
						feeds[i] = new Feed(name, url, desc, i);
					}
				}
				if (feeds != null && jEdit.getBooleanProperty(HeadlinePlugin.PROPERTY_SORT)) {
					Arrays.sort(feeds);
				}
			}
		} else {
			// create a new xml document to store the feeds
			try {
				DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
				factory.setValidating(false);
				DocumentBuilder db = factory.newDocumentBuilder();
				doc = db.newDocument();
				Node feeds = doc.createElement("feeds");
				doc.appendChild(feeds);
			} catch (ParserConfigurationException ex) {
				JOptionPane.showMessageDialog(null, "Could not initialize a new feed XML store:\n" + ex);
			}
		}
	}
	
	public void actionPerformed(java.awt.event.ActionEvent e) {
		// delete button
		if (e.getSource().equals(deleteFeed)) {
            if (feedList.getSelectedIndex() < 0) {
                return;
            }
            
			// delete the node
			int n = feeds[feedList.getSelectedIndex()].getIndex();
			
			try {
				doc.getDocumentElement().removeChild(doc.getDocumentElement().getElementsByTagName("feed").item(n));
			} catch (Exception ex) {
				JOptionPane.showMessageDialog(null, "Error deleting node " + n + " " + feeds[n].getName() + ":\n" + ex);
				return;
			}
			
			saveAndReload();
		}
		
		// add button
		if (e.getSource().equals(addFeed)) {
			updateFeed(newFeed.getText(), null, false);
		}
		
		// updateFeed button
		if (e.getSource().equals(updateFeed)) {
            if (feedList.getSelectedIndex() < 0) {
                return;
            }
            
			updateFeed(null, feeds[feedList.getSelectedIndex()], false);
		}
		
		// updateAllFeeds button
		if (e.getSource().equals(updateAllFeeds)) {
            if (feedList.getSelectedIndex() < 0) {
                return;
            }
            
			for(int i=0;i<Array.getLength(feeds);i++) {
				try {
					this.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
					updateFeed(null, feeds[i], true);
				} catch (Exception ex) {
					JOptionPane.showMessageDialog(null, "An error occurred while updating " + feeds[i].getName() + ":\n" + ex);
					break;
				} finally {
					this.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
				}
			}
			JOptionPane.showMessageDialog(null, "Feeds updated.");
		}
	}
	
	private void updateFeed(String url, Feed f, boolean quiet) {
		boolean updatemode = false;
		boolean updateanyway = false;
		boolean cont = true;
		int index = -1;
		Document d = null;
		NodeList channel = null;
		boolean atom = false; // new syndication format
        
		if (f != null) {
			url = f.getUrl();
			index = f.getIndex();
			updatemode = true;
		}
		
		String newname = null;
		String newdesc = null;
        
		if (url != "") {
			try {
				d = Headline.parseXMLFile(url, Headline.NOVALIDATE, Headline.URLMODE, Headline.NOCACHE);
			} catch (SAXException ex) {
				JOptionPane.showMessageDialog(null, url + "\n\nThe feed does not appear to be valid XML.\nThe specific error message is:\n" + ex.getMessage() + "\nThe problem is probably temporary.\nTry again later.");
				cont = false;
			} catch (ParserConfigurationException ex) {
				JOptionPane.showMessageDialog(null, url + "\n\nAn error occurred:\n" + ex.toString());
				cont = false;
			} catch (IOException ex) {
				if (ex instanceof UnknownHostException) {
					JOptionPane.showMessageDialog(null, url + "\n\nThe site could not be reached.\nIt may be temporarily offline.\nTry again later.");
				} else {
					JOptionPane.showMessageDialog(null, url + "\n\nAn error occurred:\n" + ex.toString());
				}
				cont = false;
			}
			
            // Preliminary support for Atom API feeds
            // What is Atom? http://help.blogger.com/bin/answer.py?answer=697&topic=36
            if (cont) {
				try {
					channel = d.getElementsByTagName("feed").item(0).getChildNodes();
                    atom = true;
				} catch (Exception ex) {
                    atom = false;
				}
            }
            
			if (cont && channel == null) {
				try {
					channel = d.getDocumentElement().getElementsByTagName("channel").item(0).getChildNodes();
				} catch (Exception ex) {
					JOptionPane.showMessageDialog(null, url + "\n\nThis doesn't appear to be a valid feed.\nNo channel element found.");
					cont = false;
				}
			}
			
			if (cont) {
				if (channel != null) {
					for(int i=0;i<channel.getLength();i++) {
						if (channel.item(i).getNodeName().equalsIgnoreCase("title")) {
							try {
								newname = XMLHelper.NodeText(channel.item(i));
							} catch (Exception ex) {
								cont = false;
							}
						}
                        if (atom) {
                            if (channel.item(i).getNodeName().equalsIgnoreCase("tagline")
                                || channel.item(i).getNodeName().equalsIgnoreCase("info")) {
                                    try {
                                        if (newdesc == null) { newdesc = ""; }
                                        newdesc += "\n" + XMLHelper.NodeText(channel.item(i));
                                    } catch (Exception ex) {
                                        // an empty description is valid so dont set cont=false
                                    }
                            }
                        } else {
                            if (channel.item(i).getNodeName().equalsIgnoreCase("description")) {
                                try {
                                    newdesc = XMLHelper.NodeText(channel.item(i));
                                } catch (Exception ex) {
                                    // an empty description is valid so dont set cont=false
                                }
                            }
                        }
					}
				}
			}
			
			// the feed is invalid but give the user an option to add it anyway
			if (!updatemode && newname == null) {
				if (JOptionPane.showConfirmDialog(null, "This feed does not appear to be valid.\nIt could be a temporary problem or\nthe feed might not be available anymore.\nDo you want to add it anyway?") == JOptionPane.YES_OPTION) {
					updateanyway = true;
					newname = url;
					newdesc = "This feed may be invalid but it was added anyway.  Try updating it to retrieve the true name and description.";
					cont = false;
				}
			}
			
			// show error message explaining that this feed will be skipped
			if (updatemode && newname == null) {
				JOptionPane.showMessageDialog(null, url + "\n\nThis feed could not be updated and will be skipped.");
			}
			
			if (newname != null) {
				boolean doupdate = false;
				
				// This is required in case the feed does not have a description
				if (newdesc == null) { newdesc = "No description"; }
				
				if (updatemode) {
					doupdate = true;
				} else {
					if (updateanyway) {
						doupdate = true;
					} else {
						doupdate = (JOptionPane.showConfirmDialog(null, "Add " + newname + "?") == JOptionPane.YES_OPTION);
					}
				}
				if (doupdate) {
					// add/update node
					try {
						Node newnode_feed = doc.createElement("feed");
						Node newnode_name = doc.createElement("name");
						Node newnode_desc = doc.createElement("description");
						Node newnode_url = doc.createElement("url");
						newnode_name.appendChild(doc.createTextNode(newname));
						newnode_desc.appendChild(doc.createTextNode(newdesc));
						newnode_url.appendChild(doc.createTextNode(url));
						newnode_feed.appendChild(newnode_name);
						newnode_feed.appendChild(newnode_url);
						newnode_feed.appendChild(newnode_desc);
						if (updatemode) {
							doc.getDocumentElement().replaceChild(newnode_feed, doc.getElementsByTagName("feed").item(index));
						} else {
							doc.getDocumentElement().appendChild(newnode_feed);
						}
					} catch (Exception ex) {
						if (updatemode) {
							JOptionPane.showMessageDialog(null, url + "\n\nAn error occurred trying to update the feed:\n" + ex);
							cont = false;
						} else {
							JOptionPane.showMessageDialog(null, url + "\n\nAn error occurred trying to add the feed:\n" + ex);
							cont = false;
						}
						return;
					}
					
					if (cont) {
						saveAndReload();
						
						if (!quiet) {
							if (updatemode) {
								JOptionPane.showMessageDialog(null, "Updated.");
							} else {
								JOptionPane.showMessageDialog(null, "New feed added.");
							}
						}
					}
				}
			}
		} else {
			JOptionPane.showMessageDialog(null, "You didn't enter a URL");
			return;
		}
	}
	
	private void saveAndReload() {
		// save and rebuild the dialog based on the new feeds.  this is
		// necessary because JComboBoxes cannot be edited.
		_save();
		this.removeAll();
		_init();
		this.validate();
	}
	
	public static String getFileStorePath() {
		return getFileStoreDirectory() + File.separator + getFileStoreName();
	}
	
	public static String getFileStoreDirectory() {
		StringBuffer buf = new StringBuffer();
		buf.append(jEdit.getSettingsDirectory());
		buf.append(File.separator);
		buf.append(HeadlinePlugin.STORAGE_DIRECTORY);
		return buf.toString();
	}
	
	public static String getCacheStoreDirectory() {
		StringBuffer buf = new StringBuffer();
		buf.append(jEdit.getSettingsDirectory());
		buf.append(File.separator);
		buf.append(HeadlinePlugin.CACHE_DIRECTORY);
		return buf.toString();
	}
	
	public static String getFileStoreName() {
		return HeadlinePlugin.STORAGE_FILENAME;
	}
}

