/*
 * Copyright (C) October 9, 2002 John Watson
 * john@flagrantdisregard.com
 * http://flagrantdisregard.com/headline
 *
 * $Id: XMLHelper.java,v 1.8 2004/06/10 03:23:04 yafd Exp $
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */

import java.io.*;
import javax.xml.parsers.*;
import javax.xml.transform.*;
import javax.xml.transform.dom.*;
import javax.xml.transform.stream.*;
import org.w3c.dom.*;
import org.xml.sax.*;

public class XMLHelper {
	private PrintWriter out;
	
	public XMLHelper(PrintWriter out) {
		this.out = out;
	}
	
    // Print the tree as XML
    public void printTree(Node node, String indent) {
        printTree(node, indent, false);
    }
    
    // Print the tree as text (text == true) or xml (text == false)
	public void printTree(Node node, String indent, boolean text) {
		switch (node.getNodeType()) {
			case Node.DOCUMENT_NODE:
				// print the contents of the Document node
				if (!text) { out.println("<?xml version=\"1.0\"?>\n"); }
				
				NodeList nodes = node.getChildNodes();
				if (nodes != null) {
					for (int i = 0; i < nodes.getLength(); i++) {
						printTree(nodes.item(i), "", text);
					}
				}
				
				break;
				
			case Node.ELEMENT_NODE:
				// print element and atributes
				String name = node.getNodeName();
				
				if (!text) {
                    out.print(indent + "<" + name);
                    
                    // print attributes
                    NamedNodeMap attributes = node.getAttributes();
                    for (int i = 0; i < attributes.getLength(); i++) {
                        Node current = attributes.item(i);
                        out.print(" " + current.getNodeName() + "=\"" + htmlEncode(current.getNodeValue()) +  "\"");
                    }
				}
                
				// recurse on each child
				NodeList children = node.getChildNodes();
				if (children != null) {
					if (!text) { out.println(">"); }
					for (int i = 0; i < children.getLength(); i++) {
						printTree(children.item(i), indent + "\t", text);
					}
					if (!text) { out.println(indent + "</" + name + ">"); }
				} else {
					if (!text) { out.println("/>"); }
				}
				
				break;
				
			case Node.TEXT_NODE:
			case Node.CDATA_SECTION_NODE:
				// print textual data
				if (node.getNodeValue().trim() != "") {
					if (!text) {
                        out.println(indent + htmlEncode(node.getNodeValue()));
                    } else {
                        out.print(htmlEncode(node.getNodeValue()));
                    }
				}
				
				break;
				
			case Node.PROCESSING_INSTRUCTION_NODE:
				break;
				
			case Node.ENTITY_REFERENCE_NODE:
				break;
				
			case Node.DOCUMENT_TYPE_NODE:
				break;
				
			default:
				break;
		}
	}
	
	public static String htmlEncode(String v) {
		return(v.trim().replaceAll("&", "&amp;").replaceAll("<", "&lt;").replaceAll(">", "&gt;").replaceAll("\"", "&quot;"));
	}
    
	public static String htmlDecode(String v) {
		return(v.trim().replaceAll("&quot;", "\"").replaceAll("&gt;", ">").replaceAll("&lt;", "<").replaceAll("&amp;", "&"));
	}
    
    // Converts the contents of a node to a string
    public static String NodeToString(Node n) {
        try {
            StringWriter tmpout = new StringWriter();
            PrintWriter pw = new PrintWriter(tmpout);
            XMLHelper xmh = new XMLHelper(pw);
            xmh.printTree(n, "");
            pw.close();
            tmpout.close();
            return(tmpout.toString());
        } catch (IOException ex) {
            return("");
        }
    }
    
    // Converts the contents of all the nodes in a NodeList to a string
    // This is useful for just converting all of the children of a node, for example.
    public static String NodesToString(NodeList n) {
        try {
            StringWriter tmpout = new StringWriter();
            PrintWriter pw = new PrintWriter(tmpout);
            XMLHelper xmh = new XMLHelper(pw);
            for(int i=0; i<n.getLength(); i++) {
                xmh.printTree(n.item(i), "");
            }
            pw.close();
            tmpout.close();
            return(tmpout.toString());
        } catch (IOException ex) {
            return("");
        }
    }
    
    // Strings together all of the values within a node (and subnodes) sans tags
    public static String NodeText(Node n) {
        try {
            StringWriter tmpout = new StringWriter();
            PrintWriter pw = new PrintWriter(tmpout);
            XMLHelper xmh = new XMLHelper(pw);
            xmh.printTree(n, "", true);
            pw.close();
            tmpout.close();
            return(tmpout.toString());
        } catch (IOException ex) {
            return("");
        }
    }
}

